subroutine RegressCoef()

	use Globals
	use LeastSquareModule

	implicit none


	!	common regressors
	
	real(8):: RHS(Nperiod,ncoef)
	real(8):: COVB(ncoef,ncoef), tstat(ncoef), r_sq, adjr_sq, chisq
	integer:: icoef


	!	regressors for theta

	RHS(:,1) = 1.0D0
	RHS(:,2) = dlog(Kdata)
	RHS(:,3) = dlog(Edata)
	RHS(:,4) = dlog(Pdata(Nskip+1:Nskip+Nperiod))


	!	open the log file

	open(0, file='Output/Logfile.txt', status='unknown')


	!	regressions for Theta equation with summary statistics

	call lsfit(RHS, dlog(THdata), coef_th_new, COVB, tstat, r_sq, adjr_sq, chisq)

	write(0, '(//A/)')    "Summary statistics for Theta equation            "
	write(0, '(A,F20.4)') "R^2 (in percent):                                ", r_sq*100.0D0
	write(0, '(A,F20.4)') "Adjusted R^2 (in percent):                       ", adjr_sq*100.0D0
	write(0, '(A,F20.4)') "Mean of the response (dependent) variable:       ", sum(THdata)/Nperiod

	write(0, '(A6,3(A15))') " ", "Estimate", "St. err", "t-stat"
	do icoef = 1, ncoef
	    write(0, '(A2,I2,A2,4F15.6)') "C(",icoef,")", coef_th_new(icoef), dsqrt(COVB(icoef,icoef)), tstat(icoef)
	end do


	!	regressions for law of motion for K with summary statistics

	call lsfit(RHS, dlog(KPdata), coef_K_new, COVB, tstat, r_sq, adjr_sq, chisq)

	write(0, '(//A/)')    "Summary statistics for K equation                "
	write(0, '(A,F20.4)') "R^2 (in percent):                                ", r_sq*100.0D0
	write(0, '(A,F20.4)') "Adjusted R^2 (in percent):                       ", adjr_sq*100.0D0
	write(0, '(A,F20.4)') "Mean of the response (dependent) variable:       ", sum(KPdata)/Nperiod

	write(0, '(A6,3(A15))') " ", "Estimate", "St. err", "t-stat"
	do icoef = 1, ncoef
	    write(0, '(A2,I2,A2,4F15.6)') "C(",icoef,")", coef_K_new(icoef), dsqrt(COVB(icoef,icoef)), tstat(icoef)
	end do


	!	regressions for law of motion for E with summary statistics

	call lsfit(RHS, dlog(EPdata), coef_E_new, COVB, tstat, r_sq, adjr_sq, chisq)

	write(0, '(//A/)')    "Summary statistics for E equation                "
	write(0, '(A,F20.4)') "R^2 (in percent):                                ", r_sq*100.0D0
	write(0, '(A,F20.4)') "Adjusted R^2 (in percent):                       ", adjr_sq*100.0D0
	write(0, '(A,F20.4)') "Mean of the response (dependent) variable:       ", sum(EPdata)/Nperiod

	write(0, '(A6,3(A15))') " ", "Estimate", "St. err", "t-stat"
	do icoef = 1, ncoef
	    write(0, '(A2,I2,A2,4F15.6)') "C(",icoef,")", coef_E_new(icoef), dsqrt(COVB(icoef,icoef)), tstat(icoef)
	end do


	!	save coefficients

	call SaveCoef()


end subroutine





