subroutine Simulation()

	use Globals
	use LinInterpModule
	use PolyInterpModule
	use dflib
	use Numerical_Libraries
	use rnunf_int
	use rnnor_int
	use csint_int
	use csval_int


	implicit none


	integer, parameter:: xseed = 2
	real(8):: theta_pred, pth_pred, qth_fe, theta_fe, VM, k_curr, kp_pred
	real(8):: AEkp(na), AUkp(na), Wagekp(na), EV_J(na,nk,np), EV_Jkp(na)
	real(8):: break_J(na), cscoef_J(4,na), break_wage(na), cscoef_wage(4,na)
	real(8):: ashock, eshock
	type Individual
		real(8):: A, AP, W, C, lambdashock, pthshock
		integer:: E, EP	
	end type Individual
	type(Individual), dimension(Nindiv):: Worker
	real(8), allocatable:: lambdavec(:), pthvec(:)
	real(8):: cutoff_lambda, cutoff_pth
	integer:: ind_E, ind_U, Nind_E, Nind_U


	!	expected value of matched firm: EV_J(a,K',p) = E[J(a,K',p')|p]

	do ia = 1, na
	do ik = 1, nk
	do ip = 1, np
		EV_J(ia,ik,ip) = dot_product(J(ia,ik,:), trp(ip,:))
	end do
	end do
	end do


	!	set the seed number for random number generator

	call rnset(xseed)


	!	intialize distribution of workers

	call d_rnnor(Worker%A)
	call dscal(Nindiv, std_asset_ss, Worker%A, 1)
	call dadd(Nindiv, ave_asset_ss, Worker%A, 1)

	do indiv = 1, Nindiv

		eshock = d_rnunf()

		if (eshock <= 0.94D0) then
			Worker(indiv)%E = 1
		else
			Worker(indiv)%E = 0
		end if

	end do


	!	start simulation 

	do time = 1, Nsim


		!	allocate memory for rnadom numbers
		
		Nind_E = sum(Worker%E,mask=Worker%E==1)
		Nind_U = sum(1-Worker%E,mask=Worker%E==0)

		allocate(lambdavec(Nind_E), pthvec(Nind_U))


		!	average capital of the unemployed, predicted theta and worker's matching probability

		k_curr = sum(Worker%A,mask=Worker%E==0)/Nind_U
		theta_pred = dexp(coef_th(1) + coef_th(2)*dlog(k_curr) + coef_th(3)*dlog(Pdata(time)))
		pth_pred = match_scale*theta_pred**match_pow


		!	interpolate asset decision rule at Kdata(time) and Pdata(time)

		do ia = 1, na
			AEkp(ia) = polyinterp1(k_curr, kgrid, AE(ia,:,iPdata(time)))
			AUkp(ia) = polyinterp1(k_curr, kgrid, AU(ia,:,iPdata(time)))
		end do


		!	interpolate wages at Kdata(time) and Pdata(time)

		do ia = 1, na
			Wagekp(ia) = polyinterp1(k_curr, kgrid, Wage(ia,:,iPdata(time)))
		end do

		call d_csint(agrid, Wagekp, break_wage, cscoef_wage)


		!	individual's asset holdings in the next period, assign random number for next period match status

		ind_E = 0
		ind_U = 0

		do indiv = 1, Nindiv

			if (Worker(indiv)%E == 1) then

				Worker(indiv)%W  = d_csval(Worker(indiv)%A, break_wage, cscoef_wage)
				Worker(indiv)%AP = lininterp1(Worker(indiv)%A, agrid, AEkp)

				ind_E = ind_E + 1
				lambdavec(ind_E) = d_rnunf()
				Worker(indiv)%lambdashock = lambdavec(ind_E)

			else

				Worker(indiv)%AP = lininterp1(Worker(indiv)%A, agrid, AUkp)

				ind_U = ind_U + 1
				pthvec(ind_U) = d_rnunf()
				Worker(indiv)%pthshock = pthvec(ind_U)

			end if

		end do


		!	sort lambdavec and pthvec

		call sortqq(loc(lambdavec), Nind_E, SRT$REAL8)
		call sortqq(loc(pthvec), Nind_U, SRT$REAL8)

		cutoff_lambda = lambdavec(int(lambda*Nind_E))
		cutoff_pth = pthvec(int(pth_pred*Nind_U))

		deallocate(lambdavec, pthvec)


		!	assign employment next period status

		do indiv = 1, Nindiv
	
			if (Worker(indiv)%E == 1) then
				
				if (Worker(indiv)%lambdashock <= cutoff_lambda) then
					Worker(indiv)%EP = 0
				else
					Worker(indiv)%EP = 1
				end if

			else

				if (Worker(indiv)%pthshock <= cutoff_pth) then
					Worker(indiv)%EP = 1
				else
					Worker(indiv)%EP = 0
				end if

			end if

		end do


		!	predicted average capital of the unemployed in the next period

		kp_pred = dexp(coef_K(1) + coef_K(2)*dlog(k_curr) + coef_K(3)*dlog(Pdata(time)))


		!	polynomial interpolation of EV_J at KPdata(time) and Pdata(time)

		do ia = 1, na
			EV_Jkp(ia) = polyinterp1(kp_pred, kgrid, EV_J(ia,:,iPdata(time)))
		end do


		!	spline interpolation of EV_Jkp along asset dimension

		call d_csint(agrid, EV_Jkp, break_J, cscoef_J)


		!	compute firm's value of a new match

		VM = 0.0D0

		do indiv = 1, Nindiv

			if (Worker(indiv)%E == 0) then
				VM = VM + bta*d_csval(Worker(indiv)%AP, break_J, cscoef_J)
			end if

		end do

		VM = VM/Nind_U


		!	compute qth_0 and Thetadata

		qth_fe = kappa/VM
		theta_fe = (match_scale/qth_fe)**(1.0D0/(1.0D0-match_pow))


		!	aggregate variables

		if (time > Nskip) then

			time2 = time - Nskip

			THdata(time2) = theta_fe
			Kdata(time2) = k_curr
			KPdata(time2) = sum(Worker%AP,mask=Worker%EP==0)/sum(1-Worker%EP,mask=Worker%EP==0)
			KEdata(time2) = sum(Worker%A,mask=Worker%E==1)/Nind_E
			Edata(time2) = dble(sum(Worker%E))/Nindiv
			Udata(time2) = 1.0D0-Edata(time2)
			Vdata(time2) = THdata(time2)*Udata(time2)
			EUdata(time2) = dble(sum(Worker%E,mask=Worker%EP==0))/Nindiv
			UEdata(time2) = dble(sum(Worker%EP,mask=Worker%E==0))/Nindiv
			hEUdata(time2) = EUdata(time2)/Edata(time2)
			hUEdata(time2) = UEdata(time2)/Udata(time2)
			Wdata(time2) = sum(Worker%W,mask=Worker%E==1)/Nind_E


			!	show aggregate variables on screen

			if (mod(time2,100) == 0) then
				write(*, '(A,I7)')    "time      = ", time2
				write(*, '(A,F12.5)') "Pdata     = ", Pdata(time) 
				write(*, '(A,F12.5)') "Kdata     = ", Kdata(time2)
				write(*, '(A,F12.5)') "KPdata    = ", KPdata(time2)
				write(*, '(A,F12.5)') "Udata     = ", Udata(time2) 
				write(*, '(A,F12.5)') "Vdata     = ", Vdata(time2) 
				write(*, '(A,F12.5)') "THata     = ", THdata(time2)
				write(*, '(A,F12.5)') "EUdata    = ", EUdata(time2) 
				write(*, '(A,F12.5)') "UEdata    = ", UEdata(time2) 
				write(*, '(A,F12.5)') "Wdata     = ", Wdata(time2)
				write(*, '(A)') " "
			end if


			!	save panel data

			if (panel) then

				open(100, file='Output\panel.txt', status='unknown')

				if (time2 > start_panel) then

					do indiv = 1, Nindiv_panel
				
	        			if (Worker(indiv)%E == 1) then
        	    			Worker(indiv)%C = (1.0D0+irate)*Worker(indiv)%A + Worker(indiv)%W - Worker(indiv)%AP
    			        else
            				Worker(indiv)%C = (1.0D0+irate)*Worker(indiv)%A + UI - Worker(indiv)%AP
    	        		end if
						write(100, '(I3, I6, 3F9.3, I3)')  &
						    time2-start_panel, indiv, Worker(indiv)%A, Worker(indiv)%W,	Worker(indiv)%C, Worker(indiv)%E
	
					end do
			
				end if

			end if

		end if

		!	update employment status in the previous period

		Worker%E = Worker%EP
		Worker%A = Worker%AP
		Worker%W = 0.0D0


	end do

	close(100)


end subroutine


