subroutine Simulation()

	use Globals
	use LinInterpModule
	use PolyInterpModule


	implicit none


	integer:: xseed
	real(8):: k_curr, e_curr, kp_pred, ep_pred, theta_pred, pth_pred, qth_fe, theta_fe, VM
	real(8):: AEkup(na), AUkup(na), Wagekup(na), EV_J(na,nk,ne,np), EV_Jkup(na)
	real(8):: cscoef_J(na,4), cscoef_wage(na,4)
	type Individual
		real(8):: A, AP, W, C, lambdashock, pthshock
		integer:: E, EP	
	end type Individual
	type(Individual), dimension(Nindiv):: Worker
	real(8), allocatable:: lambdavec(:), pthvec(:)
	integer, allocatable:: lambda_iperm(:), pth_iperm(:)
	real(8):: cutoff_lambda, cutoff_pth
	integer:: ind_E, ind_U, Nind_E, Nind_U, IER
	real, external:: sunif, snorm
	real(8), external:: csval
	

	!	expected value of matched firm: EV_J(a,K',p) = E[J(a,K',p')|p]

	do ia = 1, na
	do ik = 1, nk
	do ie = 1, ne
	do ip = 1, np
		EV_J(ia,ik,ie,ip) = dot_product(J(ia,ik,ie,:), trp(ip,:))
	end do
	end do
	end do
	end do


	!	set the seed number for random number generator

	xseed = 2


	!	intialize asset distribution of workers

	do indiv = 1, Nindiv
	
		Worker(indiv)%A = min(max(aveK + stdK*snorm(xseed), agrid(1)), agrid(na))

	end do


	!	initialize employment status
		
	do indiv = 1, Nindiv

		if (sunif(xseed) <= 0.94D0) then
			Worker(indiv)%E = 1
		else
			Worker(indiv)%E = 0
		end if

	end do


	!	start simulation 

	do time = 1, Nsim


		!	allocate memory for random numbers
		
		Nind_E = sum(Worker%E,mask=Worker%E==1)
		Nind_U = sum(1-Worker%E,mask=Worker%E==0)

		allocate(lambdavec(Nind_E), pthvec(Nind_U), lambda_iperm(Nind_E), pth_iperm(Nind_U))


		!	average asset, predicted theta and worker's matching probability

		k_curr = sum(Worker%A)/Nindiv
		e_curr = dble(Nind_E)/Nindiv
		theta_pred = dexp(coef_th(1) + coef_th(2)*dlog(k_curr) + coef_th(3)*dlog(e_curr) + coef_th(4)*dlog(Pdata(time)))
		pth_pred   = match_scale*theta_pred**match_pow


		!	interpolate asset decision rule at Kdata(time) and Pdata(time)

		do ia = 1, na
			AEkup(ia) = polyinterp2(k_curr, e_curr, kgrid, egrid, AE(ia,:,:,iPdata(time)))
			AUkup(ia) = polyinterp2(k_curr, e_curr, kgrid, egrid, AU(ia,:,:,iPdata(time)))
		end do


		!	interpolate wages at Kdata(time), Udata(time) and Pdata(time)

		do ia = 1, na
			Wagekup(ia) = polyinterp2(k_curr, e_curr, kgrid, egrid, Wage(ia,:,:,iPdata(time)))
		end do

		call csint(na, agrid, Wagekup, cscoef_wage)


		!	individual's asset holdings in the next period, assign random number for next period match status

		ind_E = 0
		ind_U = 0

		do indiv = 1, Nindiv

			if (Worker(indiv)%E == 1) then

				Worker(indiv)%W  = csval(na, Worker(indiv)%A, agrid, cscoef_wage)
				Worker(indiv)%AP = lininterp1(Worker(indiv)%A, agrid, AEkup)
				Worker(indiv)%C  = (1.0D0+irate)*Worker(indiv)%A + Worker(indiv)%W - Worker(indiv)%AP

				ind_E = ind_E + 1
				lambdavec(ind_E) = sunif(xseed)
				Worker(indiv)%lambdashock = lambdavec(ind_E)

			else

				Worker(indiv)%AP = lininterp1(Worker(indiv)%A, agrid, AUkup)
				Worker(indiv)%C  = (1.0D0+irate)*Worker(indiv)%A + UI - Worker(indiv)%AP

				ind_U = ind_U + 1
				pthvec(ind_U) = sunif(xseed)
				Worker(indiv)%pthshock = pthvec(ind_U)

			end if

		end do


		!	sort lambdavec and pthvec

		call sort(lambdavec, Nind_E, lambda_iperm, 2, IER)
		cutoff_lambda = lambdavec(int(lambda*Nind_E))
		call sort(pthvec, Nind_U, pth_iperm, 2, IER)
		cutoff_pth = pthvec(int(pth_pred*Nind_U))

		deallocate(lambdavec, pthvec, lambda_iperm, pth_iperm)


		!	assign employment next period status

		do indiv = 1, Nindiv
	
			if (Worker(indiv)%E == 1) then
				
				if (Worker(indiv)%lambdashock <= cutoff_lambda) then
					Worker(indiv)%EP = 0
				else
					Worker(indiv)%EP = 1
				end if

			else

				if (Worker(indiv)%pthshock <= cutoff_pth) then
					Worker(indiv)%EP = 1
				else
					Worker(indiv)%EP = 0
				end if

			end if

		end do


		!	predicted average asset and unemployment rate next period

		kp_pred = dexp(coef_K(1) + coef_K(2)*dlog(k_curr) + coef_K(3)*dlog(e_curr) + coef_K(4)*dlog(Pdata(time)))
		ep_pred = dexp(coef_E(1) + coef_E(2)*dlog(k_curr) + coef_E(3)*dlog(e_curr) + coef_E(4)*dlog(Pdata(time)))

		
		!	polynomial interpolation of EV_J at kp_pred, up_pred and Pdata(time)

		do ia = 1, na
			EV_Jkup(ia) = polyinterp2(kp_pred, ep_pred, kgrid, egrid, EV_J(ia,:,:,iPdata(time)))
		end do


		!	spline interpolation of EV_Jkp along asset dimension

		call csint(na, agrid, EV_Jkup, cscoef_J)


		!	compute firm's value of a new match

		VM = 0.0D0
		
		do indiv = 1, Nindiv

			if (Worker(indiv)%E == 0) then
				VM = VM + bta_J*csval(na, Worker(indiv)%AP, agrid, cscoef_J)
			end if

		end do

		VM = VM/Nind_U


		!	compute qth_fe and Thetadata

		qth_fe = kappa/VM
		theta_fe = (match_scale/qth_fe)**(1.0D0/(1.0D0-match_pow))


			!	show aggregate variables on screen

			if (mod(time,500) == 0) then
				write(*, '(A,I7)')    "time      = ", time
				write(*, '(A,F12.5)') "Pdata     = ", Pdata(time) 
				write(*, '(A,F12.5)') "Kdata     = ", k_curr
				write(*, '(A,F12.5)') "KPdata    = ", sum(Worker%AP)/Nindiv
				write(*, '(A,F12.5)') "KEdata    = ", sum(Worker%A,mask=Worker%E==1)/Nind_E
				write(*, '(A,F12.5)') "KUdata    = ", sum(Worker%A,mask=Worker%E==0)/Nind_U
				write(*, '(A,F12.5)') "Edata     = ", dble(Nind_E)/Nindiv
				write(*, '(A,F12.5)') "THdata    = ", theta_fe
				write(*, '(A,F12.5)') "EUdata    = ", dble(sum(Worker%E,mask=Worker%EP==0))/Nindiv
				write(*, '(A,F12.5)') "UEdata    = ", dble(sum(Worker%EP,mask=Worker%E==0))/Nindiv 
				write(*, '(A,F12.5)') "Wdata     = ", sum(Worker%W,mask=Worker%E==1)/Nind_E
				write(*, '(A)') " "
			end if




		!	aggregate variables

		if (time > Nskip) then

			time2 = time - Nskip

			THdata(time2) = theta_fe
			Kdata(time2) = k_curr
			KPdata(time2) = sum(Worker%AP)/Nindiv
			KEdata(time2) = sum(Worker%A,mask=Worker%E==1)/Nind_E
			KUdata(time2) = sum(Worker%A,mask=Worker%E==0)/Nind_U
			Cdata(time2)  = sum(Worker%C)/Nindiv
			CEdata(time2) = sum(Worker%C,mask=Worker%E==1)/Nind_E
			CUdata(time2) = sum(Worker%C,mask=Worker%E==0)/Nind_U
			K2data(time2)  = sum(Worker%A**2.0D0)/Nindiv
			KP2data(time2) = sum(Worker%AP**2.0D0)/Nindiv
			Edata(time2)  = dble(sum(Worker%E))/Nindiv
			EPdata(time2) = dble(sum(Worker%EP))/Nindiv
			Udata(time2) = 1.0D0-Edata(time2)
			Vdata(time2) = THdata(time2)*Udata(time2)
			EUdata(time2) = dble(sum(Worker%E,mask=Worker%EP==0))/Nindiv
			UEdata(time2) = dble(sum(Worker%EP,mask=Worker%E==0))/Nindiv
			hEUdata(time2) = EUdata(time2)/Edata(time2)
			hUEdata(time2) = UEdata(time2)/Udata(time2)
			Wdata(time2) = sum(Worker%W,mask=Worker%E==1)/Nind_E


			!	show aggregate variables on screen

!			if (mod(time2,500) == 0) then
!				write(*, '(A,I7)')    "time      = ", time2
!				write(*, '(A,F12.5)') "Pdata     = ", Pdata(time) 
!				write(*, '(A,F12.5)') "Kdata     = ", Kdata(time2)
!				write(*, '(A,F12.5)') "KPdata    = ", KPdata(time2)
!				write(*, '(A,F12.5)') "Edata     = ", Edata(time2) 
!				write(*, '(A,F12.5)') "Vdata     = ", Vdata(time2) 
!				write(*, '(A,F12.5)') "THata     = ", THdata(time2)
!				write(*, '(A,F12.5)') "EUdata    = ", EUdata(time2) 
!				write(*, '(A,F12.5)') "UEdata    = ", UEdata(time2) 
!				write(*, '(A,F12.5)') "Wdata     = ", Wdata(time2)
!				write(*, '(A)') " "
!			end if


			!	save panel data

			if (panel) then

				open(100, file='Output\panel.txt', status='unknown')

				if (time2 > start_panel) then

					do indiv = 1, Nindiv_panel
				
						write(100, '(I3, I6, 3F9.3, I3)')  &
						    time2-start_panel, indiv, Worker(indiv)%A, Worker(indiv)%W,	Worker(indiv)%C, Worker(indiv)%E
	
					end do
			
				end if

			end if

		end if

		!	update employment status in the previous period

		Worker%E = Worker%EP
		Worker%A = Worker%AP
		Worker%W = 0.0D0


	end do

	close(100)


end subroutine


