!	----------------------------------------------------------------------
!	Module name : ValueModule.f90
!	----------------------------------------------------------------------	*/


module ValueModule

	use Globals
	use OptimizationMod
	use PolyInterpModule

	implicit none

	real(8):: cscoef(na,4)
	real(8):: EV_W(na), EV_U(na), Wkpup(na,np), Ukpup(na,np)
	real(8), external:: csval


contains


subroutine SolveValueFunctions()


	real(8):: NewW(na,nk,ne,np), NewU(na,nk,ne,np), xmin, lbd, ubd


	!	start iteration

	err_value = 1.0D0
	iter_value = 0

	do while (err_value > tol_value)


		do ip = 1, np
		do ie = 1, ne
		do ik = 1, nk

			!	expected values in the next period according to the current employment status
	
			call ExpectedValueFunctions()


			!	employed worker's utility maximization

			call csint(na, agrid, EV_W, cscoef)

			do ia = 1, na

				income = (1.0D0+irate)*agrid(ia) + Wage(ia,ik,ie,ip)
				lbd = amin
				ubd = income
				util_leisure = 0.0D0
				call fmin(NegValueF, lbd, ubd, xmin)
				AE(ia,ik,ie,ip) = xmin
				CE(ia,ik,ie,ip) = income - AE(ia,ik,ie,ip)
				NewW(ia,ik,ie,ip) = ValueF(AE(ia,ik,ie,ip))
		
			end do


			!	unemployed worker's utility maximization

			call csint(na, agrid, EV_U, cscoef)

			do ia = 1, na

				income = (1.0D0+irate)*agrid(ia) + UI
				lbd = amin
				ubd = income
				util_leisure = B_leisure
				call fmin(NegValueF, lbd, ubd, xmin)
				AU(ia,ik,ie,ip) = xmin
				CU(ia,ik,ie,ip) = income - AU(ia,ik,ie,ip)
				NewU(ia,ik,ie,ip) = ValueF(AU(ia,ik,ie,ip))

			end do

		end do
		end do
		end do


		!	calculate improvements of value functions

		err_W = maxval(dabs(NewW - W))
		err_U = maxval(dabs(NewU - U))
		err_value = max(err_W, err_U)


		!	update value functions

		W = NewW
		U = NewU
		
		iter_value = iter_value + 1


		!	show progress

		if (mod(iter_value,200) == 0) then
			write(*, '(A,I6)') "SolveValueFunction iteration ", iter_value
			write(*, '(2(A,D12.5)/)') "err_W = ", err_W, "   err_U = ", err_U
		end if

	end do


	write(*, '(A,I6/)') "Value functions have converged at iteration " ,iter_value
	

end subroutine



real(8) function Utility(ap)


	real(8), parameter:: tiny = 1.0D-20
	real(8):: cons, uh, ap

	cons = max(income - ap, tiny)

	if (gama == 1.0D0) then
		Utility = dlog(cons) + util_leisure
	else
		Utility = (cons**(1.0D0-gama) - 1.0D0)/(1.0D0-gama) + util_leisure
	end if


end function



real(8) function ValueF(ap)


	real(8):: ap

	ValueF = Utility(ap) + bta*csval(na, ap, agrid, cscoef)

end function



real(8) function NegValueF(ap)


	real(8):: ap

	NegValueF = -ValueF(ap)

end function



subroutine ExpectedValueFunctions()


	!	polynomial interpolation at kp(ik,iu,ip) and up(ik,iu,ip)

	do jp = 1, np
	do ia = 1, na
		Wkpup(ia,jp) = polyinterp2(kp(ik,ie,ip), ep(ik,ie,ip), kgrid, egrid, W(ia,:,:,jp))
		Ukpup(ia,jp) = polyinterp2(kp(ik,ie,ip), ep(ik,ie,ip), kgrid, egrid, U(ia,:,:,jp))
	end do
	end do


	!	expected values

	do ia = 1, na
		EV_W(ia) = dot_product(((1.0D0-lambda)*Wkpup(ia,:) + lambda*Ukpup(ia,:)), trP(ip,:))
		EV_U(ia) = dot_product((pth(ik,ie,ip)*Wkpup(ia,:) + (1.0D0-pth(ik,ie,ip))*Ukpup(ia,:)), trP(ip,:))
	end do


end subroutine



end module
