module WageModule


	use Globals
	use ValueModule
	use OptimizationMod

	implicit none

	real(8):: cscoef_J(na,4)
	real(8):: EJH(na,nk,ne,np), Jkpup(na,np)


contains


subroutine EquilibriumWage()


	real(8):: NewWage(na,nk,ne,np)


	!	start iterations

	err_wage = 1.0D0
	iter_wage = 0

	do while (err_wage > tol_wage)


		!	compute value functions given wages

		call SolveValueFunctions()


		!	compute firm's value using Nash bargaining solution: J = (1.0D0-alpha)/alpha*(W - U)*ce

		J = (1.0D0-alpha)/alpha*(W-U)*CE


		!	expected value

		EJH = 0.0D0

		do ip = 1, np
		do ie = 1, ne
		do ik = 1, nk

			!	polynomial interpolate J at kp(ik,iu,ip)

			do ia = 1, na
				
				do jp = 1, np
					Jkpup(ia,jp) = polyinterp2(kp(ik,ie,ip), ep(ik,ie,ip), kgrid, egrid, J(ia,:,:,jp))
				end do
				
				EJH(ia,ik,ie,ip) = dot_product(Jkpup(ia,:), trP(ip,:))
			
			end do

		end do
		end do
		end do


		!	update wages using firm's value function

		do ip = 1, np
		do ie = 1, ne
		do ik = 1, nk

			call csint(na, agrid, EJH(:,ik,ie,ip), cscoef_J)

			do ia = 1, na
				NewWage(ia,ik,ie,ip) = Prod*pgrid(ip) - J(ia,ik,ie,ip) + bta_J*(1.0D0-lambda)*csval(na, AE(ia,ik,ie,ip), agrid, cscoef_J)
			end do

		end do
		end do
		end do
		
		
		!	compute improvement of wages

		err_wage = maxval(dabs(NewWage - Wage))

		
		!	update wages

		Wage = sfac_wage*NewWage + (1.0D0-sfac_wage)*Wage

		iter_wage = iter_wage + 1


		!	show progress

		write(*, '(A,I4)') "EquilibriumWage iteration ", iter_wage
		write(*, '(A,D12.6/)') "err_wage = ", err_wage


	end do


	!	print the final iteration number

	write(*, '(A,I4/)') "Wages have converged at iteration " ,iter_wage


end subroutine



end module
