!	----------------------------------------------------------------------
!	File name: Globals.f90
!
!	Defines global variables.
!	----------------------------------------------------------------------


module Globals

	implicit none

	save


	!	convergence check

	real(8), parameter:: tol_value = 1.0D-4, tol_wage = 1.0D-4, tol_th = 1.0D-2, tol_K = 1.0D-2, tol_E = 1.0D-2
	real(8):: err_value, err_W, err_U, err_wage, err_th, err_K, err_E, sfac_wage, sfac_value, sfac_th, sfac_K, sfac_E

	!	grids for aggregate states: productivity, average asset and unemployment rate

	integer, parameter:: nk = 5, ne = 5, np = 7
	real(8):: kgrid(nk), egrid(ne), pgrid(np)
	real(8):: aveK, stdK, kmin, kmax, aveE, emin, emax, pmin, pmax, step


	!	grids for individual asset holdings

	integer, parameter:: na = 41
	real(8):: agrid(na), amin, amax
	

	!	grids for idiosyncratic productivity

	integer, parameter:: nx = 31
	integer, parameter:: prod_new_match = nx/2+1
	real(8), parameter:: xbnd = 2.0D0
	real(8):: xgrid(nx), lxgrid(nx), mean_log_x, sigx, lxmax, lxmin


	!	average asset next period, theta, matching probabilities

	real(8):: kp(nk,ne,np), ep(nk,ne,np), Vac(nk,ne,np), theta(nk,ne,np), pth(nk,ne,np), qth(nk,ne,np)

	
	!	coefficients of theta equationa and law of motion for aggregate capital

	integer, parameter:: ncoef = 4
	real(8):: coef_th(ncoef), coef_K(ncoef), coef_E(ncoef), coef_th_new(ncoef), coef_K_new(ncoef), coef_E_new(ncoef)


	!	storage for wages, value functions and decision rules

	real(8):: Wage(na,nx,nk,ne,np)
	real(8):: W(na,nx,nk,ne,np), U(na,nk,ne,np), J(na,nx,nk,ne,np)
	real(8):: AE(na,nx,nk,ne,np), AU(na,nk,ne,np), CE(na,nx,nk,ne,np), CU(na,nk,ne,np)
	

	!	model parameters

	real(8):: Prod, irate, alpha, bta, gama, lambda, UI, UI_scale, UI_pow, B_leisure, kappa, kappa_scale, kappa_pow,	&
			   match_scale, match_pow, log_mean_x, sigex, rhox, rhop, sigep
			   

	!	auxiliary variables
	
	real(8):: income, util_leisure, bta_J, sigp
	

	!	transition probability matrix for idiosyncratic productivity

	real(8):: trX(nx,nx), prX(nx), CtrX(nx,nx), CprX(nx)


	!	transition probability matrix for aggregate productivity

	real(8):: trP(np,np), CtrP(np,np)


	!	simulated time series data

	integer, parameter:: Nindiv = 400000, Nperiod = 9000, Nskip = 3000, Nsim = Nperiod + Nskip, Nindiv_panel = 2000, start_panel = 8640
	integer, dimension(Nsim+1):: iPdata
	real(8), dimension(Nsim+1):: Pdata
	real(8), dimension(Nperiod):: THdata, KPdata, Kdata, KP2data, K2data, KEdata, KUdata, Cdata, CEdata, CUdata, Edata, EPdata, Udata, Vdata, EUdata, UEdata, hEUdata, hUEdata, Wdata, Ydata, Adata


	!	panel data generation

	logical:: panel


	!	indexing

	integer:: ik, ie, ip, jp, ia, ix, jx, iter_value, iter_wage, iter_coef, time, time2, indiv


	!	timer

	character(8):: startdate, finishdate, wage_date, sim_date
	character(10):: starttime, finishtime, wage_time, sim_time

	
end module
