!   This simulation routine works with InitializeAggregateShockSeries() 
!   that generates aggregate productivity shocks on pgrid.

subroutine Simulation()

	use Globals
	use LinInterpModule
	use PolyInterpModule


	implicit none


	integer:: xseed
	real(8):: K_curr, E_curr, Kp_pred, Ep_pred, theta_pred, pth_pred, qth_fe, theta_fe, VM
	real(8):: maxJ(na,nx,np), EmaxJGH(na,nk,ne,np), EmaxJGHkup(na)
	real(8):: Wkup(na,nx), Ukup(na), AEkup(na,nx), AUkup(na), Wagekup(na,nx)
	real(8):: cscoef_EmaxJ(na,4), cscoef_wage(na,4,nx), cscoef_Wkup(na,4,nx), cscoef_Ukup(4,na)
	type Individual
		real(8):: A, AP, W, C, lambdashock, pthshock
		integer:: E, EP, X, XP
	end type Individual
	type(Individual), dimension(Nindiv):: Worker
	real(8), allocatable:: lambdavec(:), pthvec(:)
	integer, allocatable:: lambda_iperm(:), pth_iperm(:)
	real(8):: cutoff_lambda, cutoff_pth, xshock
	integer:: ind_E, ind_U, Nind_E, Nind_U, IER
	real, external:: sunif, snorm
	real(8), external:: csval


	!	expected value of matched firm: EmaxJGH(a,K,p), used for evaluating the free-entry condition

	EmaxJGH = 0.0D0

	do ip = 1, np
	do ie = 1, ne
	do ik = 1, nk
	
		!	polynomial interpolate J at kp(ik,iu,ip) and up(ik,iu,ip), and compute max{J,0} in the next period

		do ia = 1, na
			
			do jp = 1, np
			do jx = 1, nx
				maxJ(ia,jx,jp) = max(polyinterp2(kp(ik,ie,ip), ep(ik,ie,ip), kgrid, egrid, J(ia,jx,:,:,jp)), 0.0D0)
			end do
			end do
		

			!	EmaxJGH(ia,ik,iu,ip)

			do jp = 1, np
			do jx = 1, nx
				EmaxJGH(ia,ik,ie,ip) = EmaxJGH(ia,ik,ie,ip) + maxJ(ia,jx,jp)*prX(jx)*trP(ip,jp)
			end do
			end do

		end do

	end do
	end do
	end do


	!	set the seed number for random number generator

	xseed = 2


	!	intialize asset distribution of workers

	do indiv = 1, Nindiv
	
		Worker(indiv)%A = min(max(aveK + stdK*snorm(xseed), agrid(1)), agrid(na))

	end do


    !   initialize employment status
    
    do indiv = 1, Nindiv
    
        xshock = sunif(xseed)
        if (xshock <= aveE) then
            Worker(indiv)%E = 1
        else
            Worker(indiv)%E = 0
        end if
        
    end do
    

    !   initialize productivity
    
	do indiv = 1, Nindiv
	
	    if (Worker(indiv)%E == 1) then
	    
	        xshock = sunif(xseed)
    		do ix = 1, nx
                if (xshock <= CprX(ix)) then
                    Worker(indiv)%X = ix
                    exit
                end if
            end do

        end if

	end do


	!	start simulation 

	do time = 1, Nsim


		!	allocate memory for random numbers
		
		Nind_E = sum(Worker%E,mask=Worker%E==1)
		Nind_U = sum(1-Worker%E,mask=Worker%E==0)

		allocate(lambdavec(Nind_E), pthvec(Nind_U), lambda_iperm(Nind_E), pth_iperm(Nind_U))


		!	average capital of the unemployed, predicted theta and worker's matching probability
        
		K_curr = sum(Worker%A)/Nindiv
		E_curr = dble(Nind_E)/Nindiv
		theta_pred = dexp(coef_th(1) + coef_th(2)*dlog(K_curr) + coef_th(3)*dlog(E_curr) + coef_th(4)*dlog(Pdata(time)))
		pth_pred = match_scale*theta_pred**match_pow


		!	interpolate asset decision rule at K_curr and Pdata(time)

		do ia = 1, na
			do ix = 1, nx
				AEkup(ia,ix) = polyinterp2(K_curr, E_curr, kgrid, egrid, AE(ia,ix,:,:,iPdata(time)))
			end do
			AUkup(ia) = polyinterp2(K_curr, E_curr, kgrid, egrid, AU(ia,:,:,iPdata(time)))
		end do


		!	interpolate wages at Kdata(time) and Pdata(time)

		do ia = 1, na
		do ix = 1, nx
			Wagekup(ia,ix) = polyinterp2(K_curr, E_curr, kgrid, egrid, Wage(ia,ix,:,:,iPdata(time)))
		end do
		end do

		do ix = 1, nx
			call csint(na, agrid, Wagekup(:,ix), cscoef_wage(:,:,ix))
		end do


		!	individual's asset holdings in the next period, assign random number for next period match status

		ind_E = 0
		ind_U = 0

		do indiv = 1, Nindiv

			if (Worker(indiv)%E == 1) then

				Worker(indiv)%W  = csval(na, Worker(indiv)%A, agrid, cscoef_wage(:,:,Worker(indiv)%X))
				Worker(indiv)%AP = min(max(lininterp1(Worker(indiv)%A, agrid, AEkup(:,Worker(indiv)%X)), agrid(1)), agrid(na))
       			Worker(indiv)%C  = (1.0D0+irate)*Worker(indiv)%A + Worker(indiv)%W - Worker(indiv)%AP

				ind_E = ind_E + 1
				lambdavec(ind_E) = sunif(xseed)
				Worker(indiv)%lambdashock = lambdavec(ind_E)

			else

				Worker(indiv)%AP = min(max(lininterp1(Worker(indiv)%A, agrid, AUkup), agrid(1)), agrid(na))
       			Worker(indiv)%C  = (1.0D0+irate)*Worker(indiv)%A + UI - Worker(indiv)%AP

				ind_U = ind_U + 1
				pthvec(ind_U) = sunif(xseed)
				Worker(indiv)%pthshock = pthvec(ind_U)

			end if

		end do


		!	sort lambdavec and pthvec

		call sort(lambdavec, Nind_E, lambda_iperm, 2, IER)
		cutoff_lambda = lambdavec(int(lambda*Nind_E))
		call sort(pthvec, Nind_U, pth_iperm, 2, IER)
		cutoff_pth = pthvec(int(pth_pred*Nind_U))

		deallocate(lambdavec, pthvec, lambda_iperm, pth_iperm)


		!	assign match status in the next period

		do indiv = 1, Nindiv
	
			if (Worker(indiv)%E == 1) then
				
				if (Worker(indiv)%lambdashock <= cutoff_lambda) then
					Worker(indiv)%XP = -9
				else
        	        xshock = sunif(xseed)
    	        	do ix = 1, nx
                        if (xshock <= CtrX(Worker(indiv)%X,ix)) then
                            Worker(indiv)%XP = ix
                            exit
                        end if
                    end do
				end if

			else

				if (Worker(indiv)%pthshock <= cutoff_pth) then
				    Worker(indiv)%XP = prod_new_match
				else
					Worker(indiv)%XP = -9
				end if

			end if

		end do


        !   compute aggregatge capital of the unemployed in the next period using prediction equation
        
		Kp_pred = dexp(coef_K(1) + coef_K(2)*dlog(K_curr) + coef_K(3)*dlog(E_curr) + coef_K(4)*dlog(Pdata(time)))
		Ep_pred = dexp(coef_E(1) + coef_E(2)*dlog(K_curr) + coef_E(3)*dlog(E_curr) + coef_E(4)*dlog(Pdata(time)))


		!	interpolate worker's value functions at K_next and Pdata(time+1)

		do ia = 1, na
			do ix = 1, nx
				Wkup(ia,ix) = polyinterp2(Kp_pred, Ep_pred, kgrid, egrid, W(ia,ix,:,:,iPdata(time+1)))
			end do
			Ukup(ia) = polyinterp2(Kp_pred, Ep_pred, kgrid, egrid, U(ia,:,:,iPdata(time+1)))
		end do

		do ix = 1, nx
			call csint(na, agrid, Wkup(:,ix), cscoef_Wkup(:,:,ix))
		end do
		
		call csint(na, agrid, Ukup, cscoef_Ukup)
		

        !   worker's employment decision in the next period

        do indiv = 1, Nindiv
        
			if (Worker(indiv)%XP > 0) then

				if (csval(na, Worker(indiv)%AP, agrid, cscoef_Wkup(:,:,Worker(indiv)%XP)) >= csval(na, Worker(indiv)%AP, agrid, cscoef_Ukup)) then
					Worker(indiv)%EP = 1
                else
					Worker(indiv)%EP = 0
                end if
             
            else
            
                Worker(indiv)%EP = 0   
            
            end if

        end do


		!	interpolation of EmaxJGH at K_curr, U_curr and Pdata(time)

		do ia = 1, na
			EmaxJGHkup(ia) = polyinterp2(K_curr, E_curr, kgrid, egrid, EmaxJGH(ia,:,:,iPdata(time)))
		end do

		call csint(na, agrid, EmaxJGHkup, cscoef_EmaxJ)


		!	compute firm's value of a new match

		VM = 0.0D0

		do indiv = 1, Nindiv

			if (Worker(indiv)%E == 0) then
				VM = VM + bta_J*csval(na, Worker(indiv)%AP, agrid, cscoef_EmaxJ)
			end if

		end do

		VM = VM/Nind_U


		!	compute qth_0 and Thetadata

		qth_fe = kappa/VM
		theta_fe = (match_scale/qth_fe)**(1.0D0/(1.0D0-match_pow))


		!	show aggregate variables on screen

		if (mod(time,500) == 0) then
			write(*, '(A,I7)')    "time      = ", time
			write(*, '(A,F12.5)') "Pdata     = ", Pdata(time)
			write(*, '(A,F12.5)') "Kdata     = ", K_curr
			write(*, '(A,F12.5)') "Edata     = ", dble(Nind_E)/Nindiv
			write(*, '(A,F12.5)') "Udata     = ", 1.0D0-dble(Nind_E)/Nindiv
			write(*, '(A,F12.5)') "THata     = ", theta_fe
			write(*, '(A,F12.5)') "hEUdata   = ", dble(sum(Worker%E,mask=Worker%EP==0))/Nind_E
			write(*, '(A,F12.5)') "hUEdata   = ", dble(sum(Worker%EP,mask=Worker%E==0))/Nind_U
			write(*, '(A)') " "
		end if


        !   save aggregate variables for time > Nskip
        
		if (time > Nskip) then

			time2 = time - Nskip

			THdata(time2) = theta_fe
			Kdata(time2) = K_curr
			KPdata(time2) = sum(Worker%AP)/Nindiv
			KEdata(time2) = sum(Worker%A,mask=Worker%E==1)/Nind_E
			KUdata(time2) = sum(Worker%A,mask=Worker%E==0)/Nind_U
			Cdata(time2)  = sum(Worker%C)/Nindiv
			CEdata(time2) = sum(Worker%C,mask=Worker%E==1)/Nind_E
			CUdata(time2) = sum(Worker%C,mask=Worker%E==0)/Nind_U
			K2data(time2)  = sum(Worker%A**2.0D0)/Nindiv
			KP2data(time2) = sum(Worker%AP**2.0D0)/Nindiv
			Edata(time2)  = dble(Nind_E)/Nindiv
			EPdata(time2) = dble(sum(Worker%EP,mask=Worker%EP==1))/Nindiv
			Udata(time2)  = 1.0D0-Edata(time2)
			Vdata(time2)  = THdata(time2)*Udata(time2)
			EUdata(time2) = dble(sum(Worker%E,mask=Worker%EP==0))/Nindiv
			UEdata(time2) = dble(sum(Worker%EP,mask=Worker%E==0))/Nindiv
			hEUdata(time2) = EUdata(time2)/Edata(time2)
			hUEdata(time2) = UEdata(time2)/Udata(time2)
			Wdata(time2) = sum(Worker%W,mask=Worker%E==1)/Nind_E
            Ydata(time2) = Pdata(time)*sum(xgrid(Worker%X),mask=Worker%E==1)/Nindiv
            Adata(time2) = Ydata(time2)/Edata(time2)
            

			!	save panel data

			if (panel) then

				open(100, file='Output\panel.txt', status='unknown')

				if (time2 > start_panel) then

					do indiv = 1, Nindiv_panel
						write(100,'(I3, I6, F8.3, I3, 2F7.3, I3)') time2-start_panel, indiv, Worker(indiv)%A, Worker(indiv)%X, Worker(indiv)%W, Worker(indiv)%C, Worker(indiv)%E
					end do
			
				end if

			end if

		end if


		!	update employment status in the previous period

		Worker%E = Worker%EP
		Worker%A = Worker%AP
		Worker%X = Worker%XP
		Worker%W = 0.0D0


	end do

	close(100)


end subroutine


