module WageModule


	use Globals
	use ValueModule
	use OptimizationMod


	implicit none


	real(8):: cscoef_J(na,4)
	real(8):: EmaxJFH(na,nx,nk,ne,np)


contains


subroutine EquilibriumWage()


	real(8):: NewWage(na,nx,nk,ne,np)


	!	start iterations

	err_wage = 1.0D0
	iter_wage = 0

	do while (err_wage > tol_wage)


		!	compute value functions given wages

		call SolveValueFunctions()


		!	compute firm's value using Nash bargaining solution: J = (1.0D0-alpha)/alpha*(W - U)*ce

		do ix = 1, nx
			J(:,ix,:,:,:) = (1.0D0-alpha)/alpha*(W(:,ix,:,:,:)-U)*CE(:,ix,:,:,:)
		end do


		!	expected firm's value

		call ExpectedValueFirm()


		!	update wages using firm's value function

		do ip = 1, np
		do ie = 1, ne
		do ik = 1, nk
		do ix = 1, nx

			call csint(na, agrid, EmaxJFH(:,ix,ik,ie,ip), cscoef_J)

			do ia = 1, na
				NewWage(ia,ix,ik,ie,ip) = Prod*pgrid(ip)*xgrid(ix) - J(ia,ix,ik,ie,ip) + bta_J*(1.0D0-lambda)*csval(na, AE(ia,ix,ik,ie,ip), agrid, cscoef_J)
			end do

		end do
		end do
		end do
		end do
		
		
		!	compute improvement of wages

		err_wage = maxval(dabs(NewWage - Wage))

		
		!	update wages

		Wage = sfac_wage*NewWage + (1.0D0-sfac_wage)*Wage

		iter_wage = iter_wage + 1


		!	show progress

		write(*, '(A,I4)') "EquilibriumWage iteration ", iter_wage
		write(*, '(A,D12.6/)') "err_wage = ", err_wage

	end do


	!	print the final iteration number

	write(*, '(A,I4/)') "Wages have converged at iteration " ,iter_wage


end subroutine




subroutine ExpectedValueFirm()


	real(8):: maxJ(na,nx,np)


	!	compute firm's expected value function

	EmaxJFH = 0.0D0

	do ip = 1, np
	do ie = 1, ne
	do ik = 1, nk
	
		!	polynomial interpolate J at kp(ik,iu,ip), and compute max{J,0} in the next period

		do ia = 1, na
			
			do jp = 1, np
			do jx = 1, nx
				maxJ(ia,jx,jp) = max(polyinterp2(kp(ik,ie,ip), ep(ik,ie,ip), kgrid, egrid, J(ia,jx,:,:,jp)), 0.0D0)
			end do
			end do
		

			!	EmaxJFH(ia,ix,ik,iu,ip)

			do ix = 1, nx

				do jp = 1, np
				do jx = 1, nx
					EmaxJFH(ia,ix,ik,ie,ip) = EmaxJFH(ia,ix,ik,ie,ip) + maxJ(ia,jx,jp)*trX(ix,jx)*trP(ip,jp)
				end do
				end do

			end do

		end do

	end do
	end do
	end do


end subroutine


end module
