module WageModule


	use Globals
	use InitializationModule
	use ValueModule
	use OptimizationMod
    

	implicit none

	real(8):: cscoef_J(na,4)


contains


subroutine EquilibriumWage()


	real(8):: NewWage(na,nx), maxJ(nx)


	!	initialize wage
	
	call InitializeWage()
	
	
	!	start iterations

	err_wage = 1.0D0
	iter_wage = 0

	do while (err_wage > tol_wage)


		!	initialize value functions

		call InitializeValueDecision()
	
	
		!	compute value functions given wages

		call SolveValueFunctions()


		!	update wages using Nash bargaining condition: J = (1.0D0-alpha)/alpha*(W - U)*ce

		do ia = 1, na

			
			!	J = (1.0D0-alpha)/alpha*(W - U)*ce

			J(ia,:) = (1.0D0-alpha)/alpha*(W(ia,:)-U(ia))*CE(ia,:)
			

			!	Emax[J(a,x'), 0|F]

			do jx = 1, nx
				maxJ(jx) = max(J(ia,jx), 0.0D0)
			end do

			do ix = 1, nx
				EmaxJF(ia,ix) = dot_product(maxJ, trX(ix,:))
			end do

		end do


		!	update wage

		do ix = 1, nx

			call csint(na, agrid, EmaxJF(:,ix), cscoef_J)

			do ia = 1, na

				NewWage(ia,ix) = Prod*xgrid(ix) - J(ia,ix) + bta_J*(1.0D0-lambda)*csval(na, AE(ia,ix), agrid, cscoef_J)
		
			end do

		end do


		!	compute improvement of wages

		err_wage = maxval(dabs(NewWage - Wage))

		
		!	update wages

		Wage = sfac_wage*NewWage + (1.0D0-sfac_wage)*Wage

		iter_wage = iter_wage + 1


		!	show progress

		print*, "EquilibriumWage iteration ", iter_wage
		print*, "err_wage = ", err_wage
		print*, " "

	end do


	!	print the final iteration number

	print*, "Wages have converged at iteration " ,iter_wage
	print*, " "


	!	find cutoff asset level below which workers choose to work with their idiosyncratic productivity

	do ix = 1, nx

		call csint(na, agrid, J(:,ix), cscoef_J)

		do ia = 1, dna

			DJ(ia,ix) = csval(na, dagrid(ia), agrid, cscoef_J)

		end do

		if (DJ(1,ix) < 0.0D0) then
			cutoff_a(ix) = 1
		else if (DJ(dna,ix) > 0.0D0) then
			cutoff_a(ix) = dna+1
		else
			do ia = 2, dna
				if (DJ(ia,ix) < 0.0D0) then
					cutoff_a(ix) = ia
					exit
				end if
			end do
		end if

	end do


	!	find cutoff x above which workers choose to work with their asset holdings
	
	do ia = 1, dna
	do ix = 1, nx
	
		if (DJ(ia,ix) >= 0.0D0) then
			cutoff_x(ia) = ix
			exit
		end if
		
	end do
	end do


	!	save DJ

	open(1, file='Output\DJ.txt', status='unknown')
	write(1, '(<nx>F12.6)') ((DJ(ia,ix), ix=1,nx), ia=1,dna)
	close(1)


	!	save cutoff assets

	open(1, file='Output\cutoff_asset.txt', status='unknown')
	write(1, '(I4)') cutoff_a
	close(1)

	open(1, file='Output\cutoff_xprod.txt', status='unknown')
	write(1, '(I4)') cutoff_x
	close(1)


end subroutine


end module
