!	----------------------------------------------------------------------
!	Module name: InitializationModule.f90
!	----------------------------------------------------------------------


module InitializationModule

	use Globals
	use OptimizationMod

	implicit none


contains


subroutine InitializeParameters()


	!	read parameters

	open(1, file='Input\Parameters.txt', status = 'old')

	read(1, *) Prod
	read(1, *) irate
	read(1, *) alpha
	read(1, *) bta
	read(1, *) gama
	read(1, *) lambda
	read(1, *) UI
	read(1, *) B_leisure
	read(1, *) kappa
	read(1, *) match_pow
	read(1, *) match_scale
	read(1, *) theta
	read(1, *) amin
	read(1, *) amax


	!	monthly interest rate and depreciation rate
	
	irate = (1.0D0+irate)**(1.0D0/12.0D0) - 1.0D0
	bta_J = 1.0D0/(1.0D0+irate)
	
	
	!	compute matching probability

	pth = match_scale*theta**match_pow


end subroutine



subroutine InitializeAgrid()

	implicit none

	
	!	construct unequally spaced asset grids

	step_a = (amax-amin)/(na-1)

	agrid(1) = amin
	do ia = 2, na
		agrid(ia) = agrid(ia-1) + step_a
	end do


	! save grid

	open(1, file='Output\agrid.txt', status='unknown')
	write(1, '(f12.6)') agrid

end subroutine



subroutine InitializeWageValueDecision()


	real(8):: betapth, Util_W, Util_U


	!	initialize value functions and Nash-bargaining wages
	!	compute Nash-bargaining wage for a assuming that a is fixed.

	betapth = 1.0D0/(1.0D0-bta*(1.0D0-lambda)+bta*pth)

	do ia = 1, na

		!	initialize asset decision rules
				
		AE(ia) = agrid(ia)
		AU(ia) = agrid(ia)


		!	solve Nash-bargaining wages given AE and AU

		Wage(ia) = Prod

		!	initialize value functions given AE, AU and Wage

		Util_W = dlog(irate*agrid(ia)+Wage(ia)) 
		Util_U = dlog(irate*agrid(ia)+UI) + B_leisure
		
		W(ia) = (Util_W - bta*lambda*betapth*(Util_W-Util_U))/(1.0D0-bta)
		U(ia) = (Util_U + bta*pth*betapth*(Util_W-Util_U))/(1.0D0-bta)
		J(ia) = (Prod-Wage(ia))/(1.0D0-bta_J*(1.0D0-lambda))

	end do


end subroutine



end module
