!	----------------------------------------------------------------------
!	MeasureModule.f90
!	----------------------------------------------------------------------


module MeasureModule

	use Globals
	use LinInterpModule

	implicit none


	real(8):: invAE(dna), invAU(dna), muhat(dna), phihat(dna)


contains


subroutine InitializeDAgrid()


	!	construct equally spaced asset grids that is used to find invaraint measures

	step_a = (amax-amin)/(dna-1)

	dagrid(1) = amin
	do ia = 2, dna
		dagrid(ia) = dagrid(ia-1) + step_a
	end do


	! save grid

	open(1, file='Output\dagrid.txt', status='unknown')
	write(1, '(f12.6)') dagrid

end subroutine



subroutine InitializeMeasures()

	implicit none

	integer i, j

	
	! initialize the cumulative measure of workers over assets

	do ia = 1, dna
		mu(ia) = (dagrid(ia)-dagrid(1))/(dagrid(dna)-dagrid(1))
	end do

	phi = 0.06D0*mu
	mu = 0.94D0*mu
			

end subroutine



subroutine InverseAsseetDecisions()


	!	compute inverse asset decision for employed workers

	do ia = 1, dna

		if (dagrid(ia) <= AE(na) .and. dagrid(ia) >= AE(1)) then
			invAE(ia) = lininterp1(dagrid(ia), AE, agrid)
		else if (dagrid(ia) > AE(na)) then
			invAE(ia) = dagrid(dna)
		else if (dagrid(ia) < AE(1)) then
			invAE(ia) = dagrid(1)
		end if

	end do


	!	compute inverse asset decision for unemployed workers

	do ia = 1, dna

		if (dagrid(ia) <= AU(na) .and. dagrid(ia) >= AU(1)) then
			invAU(ia) = lininterp1(dagrid(ia), AU, agrid)
		else if (dagrid(ia) > AU(na)) then
			invAU(ia) = dagrid(dna)
		else if (dagrid(ia) < AU(1)) then
			invAU(ia) = dagrid(1)
		end if

	end do


end subroutine



subroutine InvariantMeasures()


	real(8) newmu(dna), newphi(dna)


	!	preparation for computing invariant measures

	call InitializeDAgrid()
	call InitializeMeasures()
	call InverseAsseetDecisions()


	!	start iteration

	err_measure = 1.0D0
	iter_measure = 0

	do while (err_measure > tol_measure)


		!	measures after asset decisions

		muhat = 0.0D0
		phihat = 0.0D0

		do ia = 1, dna

			muhat(ia) = lininterp1(invAE(ia), dagrid, mu)
			phihat(ia) = lininterp1(invAU(ia), dagrid, phi)

		end do

		muhat = muhat/muhat(dna)*mu(dna)
		phihat = phihat/phihat(dna)*phi(dna)

		
		!	measures after new matching and exogenous separations

		newmu = (1.0D0-lambda)*muhat + pth*phihat
		newphi = lambda*muhat + (1.0D0-pth)*phihat


		!	calculate the differences between old and new measures

		err_measure = max(maxval(dabs(newmu-mu)), maxval(dabs(newphi-phi)))


		!	update measures 

		mu = newmu
		phi = newphi

		iter_measure = iter_measure + 1


		!	show progress

		if (mod(iter_measure, 10000) == 0) then
			print*, "InvariantMeasure iteration", iter_measure
			print*, mu(dna), phi(dna)
			print*, "err_measure = ", err_measure
			print*, ""
		end if

	end do


	!	print the final iteration number

	print*, "Measures have converged at iteration", iter_measure
	print*, ""


	!	compute measures as densities

	do ia = dna, 2, -1

		mu(ia) = mu(ia) - mu(ia-1)
		phi(ia) = phi(ia) - phi(ia-1)

	end do
	

end subroutine


end module